@@ -26,7 +26,7 @@ import os
 import io
 import shutil
 
-from firewall.config import ETC_FIREWALLD
+from firewall import config
 from firewall.functions import u2b_if_py2
 from firewall.core.io.io_object import PY2, IO_Object, \
     IO_Object_ContentHandler, IO_Object_XMLGenerator, check_port, \
@@ -90,7 +90,7 @@ class Helper(IO_Object):
             raise FirewallError(errors.INVALID_IPV,
                                 "'%s' not in '%s'" % (ipv, ipvs))
 
-    def _check_config(self, config, item):
+    def _check_config(self, config, item, all_config):
         if item == "ports":
             for port in config:
                 check_port(port[0])
@@ -150,15 +150,17 @@ def helper_reader(filename, path):
     helper.check_name(helper.name)
     helper.filename = filename
     helper.path = path
-    helper.builtin = False if path.startswith(ETC_FIREWALLD) else True
+    helper.builtin = False if path.startswith(config.ETC_FIREWALLD) else True
     helper.default = helper.builtin
     handler = helper_ContentHandler(helper)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
     name = "%s/%s" % (path, filename)
-    with open(name, "r") as f:
+    with open(name, "rb") as f:
+        source = sax.InputSource(None)
+        source.setByteStream(f)
         try:
-            parser.parse(f)
+            parser.parse(source)
         except sax.SAXParseException as msg:
             raise FirewallError(errors.INVALID_HELPER,
                                 "not a valid helper file: %s" % \
@@ -184,9 +186,9 @@ def helper_writer(helper, path=None):
             log.error("Backup of file '%s' failed: %s", name, msg)
 
     dirpath = os.path.dirname(name)
-    if dirpath.startswith(ETC_FIREWALLD) and not os.path.exists(dirpath):
-        if not os.path.exists(ETC_FIREWALLD):
-            os.mkdir(ETC_FIREWALLD, 0o750)
+    if dirpath.startswith(config.ETC_FIREWALLD) and not os.path.exists(dirpath):
+        if not os.path.exists(config.ETC_FIREWALLD):
+            os.mkdir(config.ETC_FIREWALLD, 0o750)
         os.mkdir(dirpath, 0o750)
 
     f = io.open(name, mode='wt', encoding='UTF-8')
