@@ -41,7 +41,8 @@ from firewall.server.decorators import handle_exceptions, \
     dbus_handle_exceptions, dbus_service_method
 from firewall import errors
 from firewall.errors import FirewallError
-from firewall.functions import portInPortRange
+from firewall.functions import portStr, portInPortRange, coalescePortRange, \
+                               breakPortRange
 
 ############################################################################
 #
@@ -177,7 +178,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     # S E T T I N G S
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
-                         out_signature=Zone.DBUS_SIGNATURE)
+                         out_signature="(sssbsasa(ss)asba(ssss)asasasasa(ss)b)")
     @dbus_handle_exceptions
     def getSettings(self, sender=None): # pylint: disable=W0613
         """get settings for zone
@@ -191,16 +192,34 @@ class FirewallDConfigZone(slip.dbus.service.Object):
             settings = tuple(_settings)
         return settings
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
+                         out_signature="a{sv}")
+    @dbus_handle_exceptions
+    def getSettings2(self, sender=None):
+        """get settings for zone
+        """
+        log.debug1("%s.getSettings2()", self._log_prefix)
+        settings = self.config.get_zone_config_dict(self.obj)
+        if settings["target"] == DEFAULT_ZONE_TARGET:
+            settings["target"] = "default"
+        return settings
+
     def _checkDuplicateInterfacesSources(self, settings):
         """Assignment of interfaces/sources to zones is different from other
            zone settings in the sense that particular interface/zone can be
            part of only one zone. So make sure added interfaces/sources have
            not already been bound to another zone."""
-        old_settings = self.config.get_zone_config(self.obj)
-        idx_i = Zone.index_of("interfaces")
-        idx_s = Zone.index_of("sources")
-        added_ifaces = set(settings[idx_i]) - set(old_settings[idx_i])
-        added_sources = set(settings[idx_s]) - set(old_settings[idx_s])
+        old_settings = self.config.get_zone_config_dict(self.obj)
+        old_ifaces = set(old_settings["interfaces"]) if "interfaces" in old_settings else set()
+        old_sources = set(old_settings["sources"]) if "sources" in old_settings else set()
+        if isinstance(settings, tuple):
+            added_ifaces = set(settings[Zone.index_of("interfaces")]) - old_ifaces
+            added_sources = set(settings[Zone.index_of("sources")]) - old_sources
+        else: # dict
+            new_ifaces = set(settings["interfaces"]) if "interfaces" in settings else set()
+            new_sources = set(settings["sources"]) if "sources" in settings else set()
+            added_ifaces = new_ifaces - old_ifaces
+            added_sources = new_sources - old_sources
 
         for iface in added_ifaces:
             if self.parent.getZoneOfInterface(iface):
@@ -210,7 +229,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                 raise FirewallError(errors.ZONE_CONFLICT, source) # or move to new zone ?
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
-                         in_signature=Zone.DBUS_SIGNATURE)
+                         in_signature="(sssbsasa(ss)asba(ssss)asasasasa(ss)b)")
     @dbus_handle_exceptions
     def update(self, settings, sender=None):
         """update settings for zone
@@ -227,6 +246,21 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         self.obj = self.config.set_zone_config(self.obj, settings)
         self.Updated(self.obj.name)
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
+                         in_signature="a{sv}")
+    @dbus_handle_exceptions
+    def update2(self, settings, sender=None):
+        """update settings for zone
+        """
+        settings = dbus_to_python(settings)
+        log.debug1("%s.update2('...')", self._log_prefix)
+        self.parent.accessCheck(sender)
+        if "target" in settings and settings["target"] == "default":
+            settings["target"] = DEFAULT_ZONE_TARGET
+        self._checkDuplicateInterfacesSources(settings)
+        self.obj = self.config.set_zone_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE)
     @dbus_handle_exceptions
     def loadDefaults(self, sender=None):
@@ -455,10 +489,16 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) in settings[6]:
-            raise FirewallError(errors.ALREADY_ENABLED,
-                                "%s:%s" % (port, protocol))
-        settings[6].append((port,protocol))
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[6]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                raise FirewallError(errors.ALREADY_ENABLED,
+                                    "%s:%s" % (port, protocol))
+        added_ranges, removed_ranges = coalescePortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[6].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[6].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -471,9 +511,17 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) not in settings[6]:
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[6]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                break
+        else:
             raise FirewallError(errors.NOT_ENABLED, "%s:%s" % (port, protocol))
-        settings[6].remove((port,protocol))
+        added_ranges, removed_ranges = breakPortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[6].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[6].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -484,13 +532,9 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         protocol = dbus_to_python(protocol, str)
         log.debug1("%s.queryPort('%s', '%s')", self._log_prefix, port,
                    protocol)
-        if (port,protocol) in self.getSettings()[6]:
-            return True
-        else:
-            # It might be a single port query that is inside a range
-            for (_port, _protocol) in self.getSettings()[6]:
-                if portInPortRange(port, _port) and protocol == _protocol:
-                    return True
+        for (_port, _protocol) in self.getSettings()[6]:
+            if portInPortRange(port, _port) and protocol == _protocol:
+                return True
 
         return False
 
@@ -587,10 +631,16 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) in settings[14]:
-            raise FirewallError(errors.ALREADY_ENABLED,
-                                "%s:%s" % (port, protocol))
-        settings[14].append((port,protocol))
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[14]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                raise FirewallError(errors.ALREADY_ENABLED,
+                                    "%s:%s" % (port, protocol))
+        added_ranges, removed_ranges = coalescePortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[14].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[14].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -603,9 +653,17 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) not in settings[14]:
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[14]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                break
+        else:
             raise FirewallError(errors.NOT_ENABLED, "%s:%s" % (port, protocol))
-        settings[14].remove((port,protocol))
+        added_ranges, removed_ranges = breakPortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[14].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[14].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -616,7 +674,11 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         protocol = dbus_to_python(protocol, str)
         log.debug1("%s.querySourcePort('%s', '%s')", self._log_prefix, port,
                    protocol)
-        return (port,protocol) in self.getSettings()[14]
+        for (_port, _protocol) in self.getSettings()[14]:
+            if portInPortRange(port, _port) and protocol == _protocol:
+                return True
+
+        return False
 
     # icmp block
 
